@@ -1047,6 +1047,7 @@ class Application(QApplication):
             prints('Using calibre Qt style:', self.using_calibre_style)
         if self.using_calibre_style:
             self.load_calibre_style()
+            self.setStyleSheet('QTabBar::tab:selected { font-style: italic }')
         self.paletteChanged.connect(self.on_palette_change)
         self.on_palette_change()
 
@@ -1119,10 +1120,9 @@ class Application(QApplication):
         transient_scroller = 0
         if isosx:
             transient_scroller = plugins['cocoa'][0].transient_scroller()
-        try:
-            self.pi.load_style(icon_map, transient_scroller)
-        except TypeError:
-            self.pi.load_style(icon_map)
+        icon_map[QStyle.SP_CustomBase + 1] = I('close-for-light-theme.png')
+        icon_map[QStyle.SP_CustomBase + 2] = I('close-for-dark-theme.png')
+        self.pi.load_style(icon_map, transient_scroller)
 
     def _send_file_open_events(self):
         with self._file_open_lock:
