@@ -33,6 +33,7 @@ from calibre.gui2.viewer.convert_book import (
 )
 from calibre.gui2.viewer.lookup import Lookup
 from calibre.gui2.viewer.overlay import LoadingOverlay
+from calibre.gui2.viewer.search import SearchPanel
 from calibre.gui2.viewer.toc import TOC, TOCSearch, TOCView
 from calibre.gui2.viewer.toolbars import ActionsToolBar
 from calibre.gui2.viewer.web_view import (
@@ -68,6 +69,7 @@ def dock_defs():
     d(_('Table of Contents'), 'toc', Qt.LeftDockWidgetArea),
     d(_('Lookup'), 'lookup', Qt.RightDockWidgetArea),
     d(_('Bookmarks'), 'bookmarks', Qt.RightDockWidgetArea)
+    d(_('Search'), 'search', Qt.LeftDockWidgetArea)
     d(_('Inspector'), 'inspector', Qt.RightDockWidgetArea, Qt.AllDockWidgetAreas)
     return ans
 
@@ -118,6 +120,7 @@ class EbookViewer(MainWindow):
             ans.setObjectName(name)
             self.addDockWidget(area, ans)
             ans.setVisible(False)
+            ans.visibilityChanged.connect(self.dock_visibility_changed)
             return ans
 
         for dock_def in itervalues(self.dock_defs):
@@ -133,6 +136,11 @@ class EbookViewer(MainWindow):
         w.l.addWidget(self.toc), w.l.addWidget(self.toc_search), w.l.setContentsMargins(0, 0, 0, 0)
         self.toc_dock.setWidget(w)
 
+        self.search_widget = w = SearchPanel(self)
+        w.search_requested.connect(self.start_search)
+        self.search_dock.setWidget(w)
+        self.search_dock.visibilityChanged.connect(self.search_widget.visibility_changed)
+
         self.lookup_widget = w = Lookup(self)
         self.lookup_dock.visibilityChanged.connect(self.lookup_widget.visibility_changed)
         self.lookup_dock.setWidget(w)
@@ -150,6 +158,10 @@ class EbookViewer(MainWindow):
         self.web_view.cfi_changed.connect(self.cfi_changed)
         self.web_view.reload_book.connect(self.reload_book)
         self.web_view.toggle_toc.connect(self.toggle_toc)
+        self.web_view.show_search.connect(self.show_search)
+        self.web_view.find_next.connect(self.search_widget.find_next_requested)
+        self.search_widget.show_search_result.connect(self.web_view.show_search_result)
+        self.web_view.search_result_not_found.connect(self.search_widget.search_result_not_found)
         self.web_view.toggle_bookmarks.connect(self.toggle_bookmarks)
         self.web_view.toggle_inspector.connect(self.toggle_inspector)
         self.web_view.toggle_lookup.connect(self.toggle_lookup)
@@ -165,11 +177,12 @@ class EbookViewer(MainWindow):
         self.web_view.print_book.connect(self.print_book, type=Qt.QueuedConnection)
         self.web_view.reset_interface.connect(self.reset_interface, type=Qt.QueuedConnection)
         self.web_view.shortcuts_changed.connect(self.shortcuts_changed)
-        self.actions_toolbar.initialize(self.web_view)
+        self.actions_toolbar.initialize(self.web_view, self.search_dock.toggleViewAction())
         self.setCentralWidget(self.web_view)
         self.loading_overlay = LoadingOverlay(self)
         self.restore_state()
         self.actions_toolbar.update_visibility()
+        self.dock_visibility_changed()
         if continue_reading:
             self.continue_reading()
 
@@ -208,7 +221,8 @@ class EbookViewer(MainWindow):
     def set_full_screen(self, on):
         if on:
             self.maximized_at_last_fullscreen = self.isMaximized()
-            self.actions_toolbar.setVisible(False)
+            if not self.actions_toolbar.visible_in_fullscreen:
+                self.actions_toolbar.setVisible(False)
             self.showFullScreen()
         else:
             self.actions_toolbar.update_visibility()
@@ -234,6 +248,16 @@ class EbookViewer(MainWindow):
     def toggle_toc(self):
         self.toc_dock.setVisible(not self.toc_dock.isVisible())
 
+    def show_search(self):
+        self.search_dock.setVisible(True)
+        self.search_widget.focus_input()
+
+    def start_search(self, search_query):
+        name = self.web_view.current_content_file
+        if name:
+            self.search_widget.start_search(search_query, name)
+            self.web_view.setFocus(Qt.OtherFocusReason)
+
     def toggle_bookmarks(self):
         is_visible = self.bookmarks_dock.isVisible()
         self.bookmarks_dock.setVisible(not is_visible)
@@ -255,6 +279,8 @@ class EbookViewer(MainWindow):
 
     def bookmarks_edited(self, bookmarks):
         self.current_book_data['annotations_map']['bookmark'] = bookmarks
+        # annotations will be saved in book file on exit
+        self.save_annotations(in_book_file=False)
 
     def bookmark_activated(self, cfi):
         self.web_view.goto_cfi(cfi)
@@ -289,6 +315,10 @@ class EbookViewer(MainWindow):
         md.setImageData(img)
         md.setUrls([url])
         QApplication.instance().clipboard().setMimeData(md)
+
+    def dock_visibility_changed(self):
+        vmap = {dock.objectName().partition('-')[0]: dock.toggleViewAction().isChecked() for dock in self.dock_widgets}
+        self.actions_toolbar.update_dock_actions(vmap)
     # }}}
 
     # Load book {{{
@@ -307,8 +337,12 @@ class EbookViewer(MainWindow):
         from .printing import print_book
         print_book(set_book_path.pathtoebook, book_title=self.current_book_data['metadata']['title'], parent=self)
 
+    @property
+    def dock_widgets(self):
+        return self.findChildren(QDockWidget, options=Qt.FindDirectChildrenOnly)
+
     def reset_interface(self):
-        for dock in self.findChildren(QDockWidget):
+        for dock in self.dock_widgets:
             dock.setFloating(False)
             area = self.dock_defs[dock.objectName().partition('-')[0]].initial_area
             self.removeDockWidget(dock)
@@ -345,6 +379,7 @@ class EbookViewer(MainWindow):
         self.loading_overlay(_('Loading book, please wait'))
         self.save_annotations()
         self.current_book_data = {}
+        self.search_widget.clear_searches()
         t = Thread(name='LoadBook', target=self._load_ebook_worker, args=(pathtoebook, open_at, reload_book or self.force_reload))
         t.daemon = True
         t.start()
@@ -403,19 +438,24 @@ class EbookViewer(MainWindow):
         self.load_book_data()
         self.update_window_title()
         initial_cfi = self.initial_cfi_for_current_book()
-        initial_toc_node = initial_bookpos = None
+        initial_position = {'type': 'cfi', 'data': initial_cfi} if initial_cfi else None
         if open_at:
             if open_at.startswith('toc:'):
                 initial_toc_node = self.toc_model.node_id_for_text(open_at[len('toc:'):])
+                initial_position = {'type': 'toc', 'data': initial_toc_node}
             elif open_at.startswith('toc-href:'):
                 initial_toc_node = self.toc_model.node_id_for_href(open_at[len('toc-href:'):], exact=True)
+                initial_position = {'type': 'toc', 'data': initial_toc_node}
             elif open_at.startswith('toc-href-contains:'):
                 initial_toc_node = self.toc_model.node_id_for_href(open_at[len('toc-href-contains:'):], exact=False)
+                initial_position = {'type': 'toc', 'data': initial_toc_node}
             elif open_at.startswith('epubcfi(/'):
-                initial_cfi = open_at
+                initial_position = {'type': 'cfi', 'data': open_at}
+            elif open_at.startswith('ref:'):
+                initial_position = {'type': 'ref', 'data': open_at[len('ref:'):]}
             elif is_float(open_at):
-                initial_bookpos = float(open_at)
-        self.web_view.start_book_load(initial_cfi=initial_cfi, initial_toc_node=initial_toc_node, initial_bookpos=initial_bookpos)
+                initial_position = {'type': 'bookpos', 'data': float(open_at)}
+        self.web_view.start_book_load(initial_position=initial_position)
 
     def load_book_data(self):
         self.load_book_annotations()
@@ -444,7 +484,10 @@ class EbookViewer(MainWindow):
             merge_annotations(parse_annotations(raw), amap)
 
     def update_window_title(self):
-        title = self.current_book_data['metadata']['title']
+        try:
+            title = self.current_book_data['metadata']['title']
+        except Exception:
+            title = _('Unknown')
         book_format = self.current_book_data['manifest']['book_format']
         title = '{} [{}] — {}'.format(title, book_format, self.base_window_title)
         self.setWindowTitle(title)
@@ -466,14 +509,14 @@ class EbookViewer(MainWindow):
     # }}}
 
     # State serialization {{{
-    def save_annotations(self):
+    def save_annotations(self, in_book_file=True):
         if not self.current_book_data:
             return
         amap = self.current_book_data['annotations_map']
         annots = as_bytes(serialize_annotations(amap))
         with open(os.path.join(annotations_dir, self.current_book_data['annotations_path_key']), 'wb') as f:
             f.write(annots)
-        if self.current_book_data.get('pathtoebook', '').lower().endswith(
+        if in_book_file and self.current_book_data.get('pathtoebook', '').lower().endswith(
                 '.epub') and get_session_pref('save_annotations_in_ebook', default=True):
             path = self.current_book_data['pathtoebook']
             if os.access(path, os.W_OK):
@@ -490,7 +533,7 @@ class EbookViewer(MainWindow):
         state = vprefs['main_window_state']
         geom = vprefs['main_window_geometry']
         if geom and get_session_pref('remember_window_geometry', default=False):
-            self.restoreGeometry(geom)
+            QApplication.instance().safe_restore_geometry(self, geom)
         if state:
             self.restoreState(state, self.MAIN_WINDOW_STATE_VERSION)
             self.inspector_dock.setVisible(False)
@@ -500,6 +543,7 @@ class EbookViewer(MainWindow):
 
     def closeEvent(self, ev):
         self.shutting_down = True
+        self.search_widget.shutdown()
         try:
             self.save_annotations()
             self.save_state()
